local window, canvas, ctx, newImage, drawImage
window = infatuated.js.global.window
canvas = infatuated.canvas
ctx = infatuated.ctx
newImage = window.infatuated.newImage
drawImage = window.infatuated.drawImage

infatuated.font = '12px "Bitstream Vera Sans", "DejaVu Sans", sans-serif'
infatuated.fontHeight = 12

--[[
local bc = window.document:createElement("canvas")
bc.width = canvas.width
bc.height = canvas.height
local bx = bc:getContext("2d")
bx.globalCompositeOperation = "destination-atop"
infatuated.bufferCanvas = bc
infatuated.bufferContext = bx
]]

love.graphics = {}
function love.graphics.reset()
	love.graphics.setColor(1, 1, 1, 1)
	love.graphics.setBackgroundColor(0, 0, 0, 1)
	love.graphics.origin()
end
function love.graphics.clear(r, g, b, a)
	if r then
		local a, b = infatuated.parseColor(r, g, b, a)
		ctx.fillStyle = b
	else
		ctx.fillStyle = infatuated.backgroundString
	end
	ctx:fillRect(0, 0, 800, 600)
end
function love.graphics.origin(x, y)
	x, y = x or 0, y or 0
	assert(type(x) == "number", "bad argument #1 to origin (expected number)")
	assert(type(y) == "number", "bad argument #2 to origin (expected number)")
	infatuated.dx, infatuated.dy = x, y
	ctx:translate(x, y)
end
function love.graphics.present() end
function love.graphics.isActive()
	return true
end

function love.graphics.getWidth()
	return canvas.width
end
function love.graphics.getHeight()
	return canvas.height
end
function love.graphics.getDimensions()
	return canvas.width, canvas.height
end

function love.graphics.getColor()
	local c
	c = infatuated.color
	return c[1], c[2], c[3], c[4]
end
function love.graphics.setColor(r, g, b, a)
	infatuated.color, infatuated.colorString = infatuated.parseColor(r, g, b, a)
end
function love.graphics.getBackgroundColor()
	local c
	c = infatuated.background
	return c[1], c[2], c[3], c[4]
end
function love.graphics.setBackgroundColor(r, g, b, a)
	infatuated.background, infatuated.backgroundString = infatuated.parseColor(r, g, b, a)
end

function love.graphics.push() end
function love.graphics.pop() end
function love.graphics.scale() end
function love.graphics.setBlendMode() end

function love.graphics.getLineWidth()
	return infatuated.lineWidth
end
function love.graphics.setLineWidth(width)
	assert(type(width) == "number", "bad argument #1 to setLineWidth (expected number)")
	infatuated.lineWidth = width
end

local Drawable = class("Drawable", infatuated.classes.Object)
infatuated.classes.Drawable = Drawable
function Drawable:initialize()
	infatuated.classes.Object.initialize(self)
end

local Texture = class("Texture", Drawable)
function Texture:initialize()
	Drawable.initialize(self)
end
function Texture:getWidth()
	return self.image.width
end
function Texture:getHeight()
	return self.image.height
end

local Image = class("Image", Texture)
infatuated.classes.Image = Image
function Image:initialize(a)
	Texture.initialize(self)
	local reterr
	if type(a) == "string" then
		infatuated.print("Loading image from %q", a)
		local image = newImage()
		a = ("/"..a):gsub("/%.%.", "/")
		image.src = infatuated.path..a
		image.onload = function(self)
			infatuated.print("Loaded image")
			coroutine.resume(infatuated.thread)
		end
		image.onerror = function(self)
			infatuated.print("Failed to load image")
			reterr = string.format("failed to load image %q", a)
			coroutine.resume(infatuated.thread)
		end
		coroutine.yield()
		self.image = image
	else
		infatuated.print("Loading image from ImageData")
		local image = newImage()
		image.src = "data:image/png;base64,"..a.data
		image.onload = function(self)
			infatuated.print("Loaded image")
			coroutine.resume(infatuated.thread)
		end
		image.onerror = function(self)
			infatuated.print("Failed to load image")
			reterr = "failed to load image from ImageData"
			coroutine.resume(infatuated.thread)
		end
		coroutine.yield()
		self.image = image
	end
	assert(not reterr, reterr)
end
function love.graphics.newImage(a)
	local t = type(a)
	assert(t == "string" or (t == "table" and tostring(a.class) == "class ImageData"), "bad argument #1 to newImage (expected string or ImageData)")
	return Image:new(a)
end

local SpriteBatch = class("SpriteBatch", Drawable)
infatuated.classes.SpriteBatch = SpriteBatch
function SpriteBatch:initialize(image, max)
	self._max = max
end
function SpriteBatch:getBufferSize()
	return self._max
end
function SpriteBatch:setBufferSize(maxsprites)
	assert(type(maxsprites) == "number", "bad argument #1 to setBufferSize (expected number)")
	assert(maxsprites > 0, "maxsprites must be greater than 0")
	self._max = maxsprites
end
function SpriteBatch:add() end
function SpriteBatch:flush() end
function SpriteBatch:clear() end
function love.graphics.newSpriteBatch(image, maxsprites)
	maxsprites = maxsprites or 1000
	assert(type(image) == "table" and tostring(image.class) == "class Image", "bad argument #1 to newSpriteBatch (expected Image)")
	assert(type(maxsprites) == "number", "bad argument #2 to newSpriteBatch (expected number)")
	assert(maxsprites > 0, "maxsprites must be greater than 0")
	return SpriteBatch:new(image, maxsprites, usage)
end

function love.graphics.draw(obj, x, y, r, sx, sy, ox, oy)
	assert(type(obj) == "table" and obj.typeOf and obj:typeOf("Drawable"), "bad argument #1 to draw (expected Drawable)")
	if obj:typeOf("SpriteBatch") then return end
	r = r or 0
	ox = ox or 0
	oy = oy or 0
	assert(type(x) == "number", "bad argument #2 to draw (expected number)")
	assert(type(y) == "number", "bad argument #3 to draw (expected number)")
	assert(type(r) == "number", "bad argument #4 to draw (expected number)")
	assert(type(ox) == "number", "bad argument #7 to draw (expected number)")
	assert(type(oy) == "number", "bad argument #8 to draw (expected number)")
	ctx:save()
	ctx:translate(x, y)
	ctx:rotate(r)
	-- color tinting looks HARD!
	ctx.globalAlpha = infatuated.color[4]
	ctx:drawImage(obj.image, -ox, -oy)
	ctx:restore()
end
function love.graphics.print(text, x, y, r, sx, sy, ox, oy, kx, ky)
	if type(text) == "number" then text = tostring(text) end
	assert(type(text) == "string", "bad argument #1 to print (expected string)")
	assert(type(x) == "number", "bad argument #2 to print (expected number)")
	assert(type(y) == "number", "bad argument #3 to print (expected number)")
	ctx.fillStyle = infatuated.colorString
	ctx.font = infatuated.font
	local h = infatuated.fontHeight
	local i = 0
	for line in (text.."\n"):gmatch(".-\n") do
		i = i+1
		ctx:fillText(line, x, y+(h*i))
	end
end
local function findbackwards(str, match)
	for i=#str, 1, -1 do
		if str:sub(i, i) == match then
			return i
		end
	end
end
function love.graphics.printf(text, x, y, limit, align, r, sx, sy, ox, oy, kx, ky)
	if type(text) == "number" then text = tostring(text) end
	align = align or "left"
	assert(type(text) == "string", "bad argument #1 to printf (expected string)")
	assert(type(x) == "number", "bad argument #2 to printf (expected number)")
	assert(type(y) == "number", "bad argument #3 to printf (expected number)")
	assert(type(limit) == "number", "bad argument #4 to printf (expected number)")
	assert(type(align) == "string", "bad argument #5 to printf (expected string)")
	ctx.fillStyle = infatuated.colorString
	ctx.font = infatuated.font
	local h = infatuated.fontHeight
	local t = {}
	local i = 0
	for line in (text.."\n"):gmatch(".-\n") do
		i = i+1
		line = line:gsub("^ +", "")
		line = line:gsub(" +$", "")
		t[i] = line
	end
	i = 1
	local wrapped = false
	while t[i] do
		local width = ctx:measureText(t[i]).width
		while width > limit do
			width = ctx:measureText(t[i]).width
			local p = findbackwards(t[i], " ")
			if p then
				local a = t[i]:sub(0, p-1)
				local b = t[i]:sub(p+1)
				t[i] = a
				if not wrapped then
					wrapped = true
					table.insert(t, i+1, "")
				end
				t[i+1] = b.." "..t[i+1]
			end
		end
		if align == "left" then
			ctx:fillText(t[i], x, y+(h*i))
		elseif align == "center" then
			ctx:fillText(t[i], x+(limit/2)-(width/2), y+(h*i))
		elseif align == "right" then			
			ctx:fillText(t[i], x+limit-width, y+(h*i))
		end
		i = i+1
		wrapped = false
	end
end

function love.graphics.rectangle(mode, x, y, w, h)
	assert(type(x) == "number", "bad argument #2 to rectangle (expected number)")
	assert(type(y) == "number", "bad argument #3 to rectangle (expected number)")
	assert(type(w) == "number", "bad argument #4 to rectangle (expected number)")
	assert(type(h) == "number", "bad argument #5 to rectangle (expected number)")
	if mode == "fill" then
		ctx.fillStyle = infatuated.colorString
		ctx:fillRect(x, y, w, h)
	elseif mode == "line" then
		ctx.strokeStyle = infatuated.colorString
		local lw = infatuated.lineWidth
		ctx.lineWidth = lw
		ctx:strokeRect(x, y, w, h)
	else
		error("bad argument #1 to rectangle")
	end
end

