love.mouse = {}
local c = infatuated.canvas
local window = infatuated.js.global.window
local cursors = {
	-- https://love2d.org/wiki/CursorType
	-- https://developer.mozilla.org/en-US/docs/Web/CSS/cursor
	arrow = "default",
	ibeam = "text",
	wait = "wait",
	waitarrow = "progress",
	crosshair = "crosshair",
	hand = "pointer",
	sizewe = "ew-resize",
	sizens = "ns-resize",
	sizenesw = "nesw-resize",
	sizenwse = "nwse-resize",
	sizeall = "move",
	no = "not-allowed"
}
infatuated.cursors = cursors

local Cursor = class("Cursor", infatuated.classes.Object)
infatuated.classes.Cursor = Cursor
function Cursor:initialize(ctype, image)
	self._ctype = ctype
	self._style = image or cursors[ctype]
end
function Cursor:getType()
	return self._ctype
end

function love.mouse.newCursor(image, x, y)
	x, y = x or 0, y or 0
	assert(type(x) == "number", "bad argument #2 to newCursor (expected number)")
	assert(type(y) == "number", "bad argument #3 to newCursor (expected number)")
	if type(image) == "string" then
		image = "game/"..window:encodeURI(image)
		local retval, reterr
		local img = window.infatuated:newImage()
		img.onload = function(self)
			infatuated.print("Loaded cursor image")
			retval = Cursor:new("image", string.format("url(%q) %d %d, auto", image, x, y))
			coroutine.resume(infatuated.thread)
		end
		img.onerror = function(self, event)
			infatuated.print("Failed to load cursor image")
			reterr = string.format("failed to load cursor image %q", image)
			coroutine.resume(infatuated.thread)
		end
		infatuated.print("Loading cursor image %q", image)
		img.src = image
		coroutine.yield()
		assert(not reterr, reterr)
		return retval
	else
		error("bad argument #1 to newCursor (expected string)")
	end
end
function love.mouse.getSystemCursor(ctype)
	assert(type(ctype) == "string", "bad argument #1 to getSystemCursor (expected string)")
	assert(cursors[ctype], "ctype must be type of CursorType")
	return Cursor:new(ctype)
end
function love.mouse.getCursor()
	return infatuated.cursorobj
end
function love.mouse.setCursor(cursor)
	if not cursor then
		return love.mouse.setCursor(love.mouse.getSystemCursor("arrow"))
	end
	assert(type(cursor) == "table", "bad argument #1 to setCursor (expected Cursor)")
	infatuated.cursorobj = cursor
	if infatuated.cursorvisible then
		c.style.cursor = cursor._style
	end
end
love.mouse.setCursor()

function love.mouse.getVisible()
	return infatuated.cursorvisible
end
function love.mouse.setVisible(bool)
	assert(type(bool) == "boolean", "bad argument #1 to setVisible (expected boolean)")
	infatuated.cursorvisible = bool
	if bool then
		c.style.cursor = infatuated.cursorobj._style
	else
		c.style.cursor = "none"
	end
end
love.mouse.setVisible(true)

function love.mouse.getPosition()
	return infatuated.mousex, infatuated.mousey
end
function love.mouse.getX()
	return infatuated.mousex
end
function love.mouse.getY()
	return infatuated.mousey
end
infatuated.mousex, infatuated.mousey = 0, 0
c:addEventListener("mousemove", function(self, event)
	local rect = c:getBoundingClientRect()
	infatuated.mousex = event.clientX-rect.left-0.5
	infatuated.mousey = event.clientY-rect.top-0.5
end)

function love.mouse.isGrabbed()
	return infatuated.grabbed
end
function love.mouse.setGrabbed(bool)
	assert(type(bool) == "boolean", "bad argument #1 to setGrabbed (boolean expected)")
	if bool then
		local reterr
		function window.document:onpointerlockchange(event)
			infatuated.msgbox:hide()
			if window.document.pointerLockElement ~= c then
				reterr = "pointer lock denied"
			end
			coroutine.resume(infatuated.thread)
		end
		infatuated.msgbox:show("Pointer lock", "The game has attempted to grab your cursor. Click 'OK' to continue.", {"OK"}, function()
			c:requestPointerLock()
		end)
		coroutine.yield()
		if reterr then error(reterr) end
	elseif infatuated.grabbed then
		window.document:exitPointerLock()
	end
	infatuated.grabbed = bool
end
love.mouse.setGrabbed(false)
